@@ -50,6 +50,7 @@
 package com.openexchange.http.grizzly.osgi;
 
 import java.util.List;
+import java.util.TimeZone;
 import java.util.concurrent.TimeUnit;
 import javax.servlet.Filter;
 import org.glassfish.grizzly.comet.CometAddOn;
@@ -58,6 +59,7 @@ import org.glassfish.grizzly.http.server.OXErrorPageGenerator;
 import org.glassfish.grizzly.http.server.OXHttpServer;
 import org.glassfish.grizzly.http.server.OXSessionManager;
 import org.glassfish.grizzly.http.server.ServerConfiguration;
+import org.glassfish.grizzly.http.server.accesslog.AccessLogBuilder;
 import org.glassfish.grizzly.nio.transport.TCPNIOTransport;
 import org.glassfish.grizzly.nio.transport.TCPNIOTransportBuilder;
 import org.glassfish.grizzly.ssl.SSLContextConfigurator;
@@ -70,6 +72,9 @@ import com.google.common.collect.ImmutableList;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.dispatcher.DispatcherPrefixService;
 import com.openexchange.exception.OXException;
+import com.openexchange.http.grizzly.GrizzlyAccessLogConfig;
+import com.openexchange.http.grizzly.GrizzlyAccessLogConfig.Format;
+import com.openexchange.http.grizzly.GrizzlyAccessLogConfig.RotatePolicy;
 import com.openexchange.http.grizzly.GrizzlyConfig;
 import com.openexchange.http.grizzly.GrizzlyExceptionCode;
 import com.openexchange.http.grizzly.service.comet.CometContextService;
@@ -168,75 +173,140 @@ public class GrizzlyActivator extends HousekeepingActivator {
             serverConfiguration.setAllowPayloadForUndefinedHttpMethods(true);
             serverConfiguration.setDefaultErrorPageGenerator(OXErrorPageGenerator.getInstance());
 
-            final NetworkListener networkListener = new NetworkListener("http-listener", grizzlyConfig.getHttpHost(), grizzlyConfig.getHttpPort());
-
-            /*
-             * Set the maximum size of the PUT/POST body generated by an HTML form to honor "com.openexchange.servlet.maxBodySize"
-             */
-            int maxBodySize = grizzlyConfig.getMaxBodySize();
-            networkListener.setMaxFormPostSize(maxBodySize);
-            networkListener.setMaxBufferedPostSize(maxBodySize);
-            networkListener.setMaxHttpHeaderSize(grizzlyConfig.getMaxHttpHeaderSize());
-            networkListener.setSessionManager(sessionManager);
-
-            // Set the transport
-            {
-                TCPNIOTransport configuredTcpNioTransport = buildTcpNioTransport(configurationService, threadPool);
-                networkListener.setTransport(configuredTcpNioTransport);
-            }
-
-            // Configure keep-alive
-            /*-
-             * Keep default settings for now.
-             * Otherwise alter KeepAlive instance.
-             *
-             * {
-             *     final KeepAlive keepAlive = networkListener.getKeepAlive();
-             *     keepAlive.setIdleTimeoutInSeconds(-1);
-             *     keepAlive.setMaxRequestsCount(-1);
-             * }
-             *
-             */
-
+            // Check for JMX
             if (grizzlyConfig.isJMXEnabled()) {
-                grizzly.getServerConfiguration().setJmxEnabled(true);
+                serverConfiguration.setJmxEnabled(true);
                 log.info("Enabled JMX for Grizzly server.");
             }
 
-            if (grizzlyConfig.isWebsocketsEnabled()) {
-                networkListener.registerAddOn(new WebSocketAddOn());
+            // Check for access.log
+            GrizzlyAccessLogConfig accessLogConfig = grizzlyConfig.getAccessLogConfig();
+            if (accessLogConfig.isEnabled()) {
+                AccessLogBuilder builder = new AccessLogBuilder(accessLogConfig.getFile());
+
+                Format format = accessLogConfig.getFormat();
+                if (null != format) {
+                    switch (format) {
+                        case AGENT:
+                            builder.format(org.glassfish.grizzly.http.server.accesslog.ApacheLogFormat.AGENT);
+                            break;
+                        case COMBINED:
+                            builder.format(org.glassfish.grizzly.http.server.accesslog.ApacheLogFormat.COMBINED);
+                            break;
+                        case COMMON:
+                            builder.format(org.glassfish.grizzly.http.server.accesslog.ApacheLogFormat.COMMON);
+                            break;
+                        case REFERER:
+                            builder.format(org.glassfish.grizzly.http.server.accesslog.ApacheLogFormat.REFERER);
+                            break;
+                        case VHOST_COMBINED:
+                            builder.format(org.glassfish.grizzly.http.server.accesslog.ApacheLogFormat.VHOST_COMBINED);
+                            break;
+                        case VHOST_COMMON:
+                            builder.format(org.glassfish.grizzly.http.server.accesslog.ApacheLogFormat.VHOST_COMMON);
+                            break;
+                        default:
+                            break;
+                    }
+                }
+
+                RotatePolicy rotatePolicy = accessLogConfig.getRotatePolicy();
+                if (null != rotatePolicy) {
+                    switch (rotatePolicy) {
+                        case DAILY:
+                            builder.rotatedDaily();
+                            break;
+                        case HOURLY:
+                            builder.rotatedHourly();
+                            break;
+                        case NONE: /* fall-through */
+                        default:
+                            break;
+                    }
+                }
+
+                int statusThreshold = accessLogConfig.getStatusThreshold();
+                if (statusThreshold > 0) {
+                    builder.statusThreshold(statusThreshold);
+                }
+
+                TimeZone timeZone = accessLogConfig.getTimeZone();
+                if (null != timeZone) {
+                    builder.timeZone(timeZone);
+                }
+
+                builder.instrument(serverConfiguration);
+            }
+
+            // Check for Web-Sockets
+            boolean websocketsEnabled = grizzlyConfig.isWebsocketsEnabled();
+            if (websocketsEnabled) {
                 registerService(WebApplicationService.class, new WebApplicationServiceImpl());
                 log.info("Enabled WebSockets for Grizzly server.");
             }
 
-            if (grizzlyConfig.isCometEnabled()) {
-                networkListener.registerAddOn(new CometAddOn());
+            // Check for Comet
+            boolean cometEnabled = grizzlyConfig.isCometEnabled();
+            if (cometEnabled) {
                 registerService(CometContextService.class, new CometContextServiceImpl());
                 log.info("Enabled Comet for Grizzly server.");
             }
 
-            grizzly.addListener(networkListener);
-            log.info("Prepared Grizzly HttpNetworkListener on host: {} and port: {}, but not yet started...", grizzlyConfig.getHttpHost(), Integer.valueOf(grizzlyConfig.getHttpPort()));
+            // Initialize HTTP network listener
+            {
+                NetworkListener networkListener = new NetworkListener("http-listener", grizzlyConfig.getHttpHost(), grizzlyConfig.getHttpPort());
 
+                // Set the maximum body/header size as well as HTTP session manager
+                networkListener.setMaxFormPostSize(grizzlyConfig.getMaxBodySize());
+                networkListener.setMaxBufferedPostSize(grizzlyConfig.getMaxBodySize());
+                networkListener.setMaxHttpHeaderSize(grizzlyConfig.getMaxHttpHeaderSize());
+                networkListener.setSessionManager(sessionManager);
+
+                // Set the transport
+                networkListener.setTransport(buildTcpNioTransport(configurationService, threadPool));
+
+                // Web Socket and Comet
+                if (websocketsEnabled) {
+                    networkListener.registerAddOn(new WebSocketAddOn());
+                }
+                if (cometEnabled) {
+                    networkListener.registerAddOn(new CometAddOn());
+                }
+
+                // Add HTTP network listener to Grizzly server
+                grizzly.addListener(networkListener);
+                log.info("Prepared Grizzly HTTP network listener on host: {} and port: {}, but not yet started...", grizzlyConfig.getHttpHost(), Integer.valueOf(grizzlyConfig.getHttpPort()));
+            }
+
+
+            // Initialize HTTPS network listener (if enabled)
             if (grizzlyConfig.isSslEnabled()) {
                 NetworkListener networkSslListener = new NetworkListener("https-listener", grizzlyConfig.getHttpHost(), grizzlyConfig.getHttpsPort());
-                networkSslListener.setMaxFormPostSize(maxBodySize);
-                networkSslListener.setMaxBufferedPostSize(maxBodySize);
+
+                // Set the maximum body/header size as well as HTTP session manager
+                networkSslListener.setMaxFormPostSize(grizzlyConfig.getMaxBodySize());
+                networkSslListener.setMaxBufferedPostSize(grizzlyConfig.getMaxBodySize());
                 networkSslListener.setMaxHttpHeaderSize(grizzlyConfig.getMaxHttpHeaderSize());
                 networkSslListener.setSessionManager(sessionManager);
+
+                // Configure and enabled SSL
                 networkSslListener.setSSLEngineConfig(createSslConfiguration(grizzlyConfig));
                 networkSslListener.setSecure(true);
-                TCPNIOTransport configuredTcpNioTransportSsl = buildTcpNioTransport(configurationService, threadPool);
-                networkSslListener.setTransport(configuredTcpNioTransportSsl);
-                if (grizzlyConfig.isWebsocketsEnabled()) {
+
+                // Set the transport
+                networkSslListener.setTransport(buildTcpNioTransport(configurationService, threadPool));
+
+                // Web Socket and Comet
+                if (websocketsEnabled) {
                     networkSslListener.registerAddOn(new WebSocketAddOn());
                 }
-                if (grizzlyConfig.isCometEnabled()) {
+                if (cometEnabled) {
                     networkSslListener.registerAddOn(new CometAddOn());
                 }
-                log.info("Enabled SSL for Grizzly server.");
+
+                // Add HTTPS network listener to Grizzly server
                 grizzly.addListener(networkSslListener);
-                log.info("Prepared secure Grizzly HttpNetworkListener on host: {} and port: {}, but not yet started...", grizzlyConfig.getHttpHost(), Integer.valueOf(grizzlyConfig.getHttpsPort()));
+                log.info("Prepared Grizzly HTTPS network listener on host: {} and port: {}, but not yet started...", grizzlyConfig.getHttpHost(), Integer.valueOf(grizzlyConfig.getHttpsPort()));
             }
 
             // Start Grizzly server
@@ -254,7 +324,7 @@ public class GrizzlyActivator extends HousekeepingActivator {
                 }
 
                 // Create the HttpService factory. Each distinct bundle will get its own instance of HttpServiceImpl.
-                httpServiceFactory = new HttpServiceFactory(grizzly, builder.build(), context.getBundle());
+                httpServiceFactory = new HttpServiceFactory(grizzly, builder.build(), grizzlyConfig.isSupportHierachicalLookupOnNotFound(), grizzlyConfig.getCookieMaxInactivityInterval(), context.getBundle());
             }
 
             // Initialize the filter tracker
