@@ -64,7 +64,6 @@ import javax.servlet.ServletException;
 import javax.servlet.ServletRequest;
 import javax.servlet.ServletResponse;
 import org.osgi.framework.BundleContext;
-import org.osgi.framework.Constants;
 import org.osgi.framework.ServiceReference;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import org.slf4j.Logger;
@@ -301,46 +300,6 @@ public class ServletFilterTracker implements ServiceTrackerCustomizer<Filter, Fi
         }
     }
 
-    /**
-     * Gets the service ranking by look-up of <code>"service.ranking"</code> property.
-     * <p>
-     * See {@link Constants#SERVICE_RANKING}.
-     *
-     * @param reference The service reference providing properties Dictionary object of the service
-     * @return The ranking or <code>0</code> (zero) if absent
-     */
-    private <S> int getRanking(final ServiceReference<S> reference) {
-        return getRanking(reference, 0);
-    }
-
-    /**
-     * Gets the service ranking by look-up of <code>"service.ranking"</code> property.
-     * <p>
-     * See {@link Constants#SERVICE_RANKING}.
-     *
-     * @param reference The service reference providing properties Dictionary object of the service
-     * @param defaultRanking The default ranking if {@link Constants#SERVICE_RANKING} property is absent
-     * @return The ranking or <code>0</code> (zero) if absent
-     */
-    private <S> int getRanking(final ServiceReference<S> reference, final int defaultRanking) {
-        int ranking = defaultRanking;
-        {
-            final Object oRanking = reference.getProperty(Constants.SERVICE_RANKING);
-            if (null != oRanking) {
-                if (oRanking instanceof Integer) {
-                    ranking = ((Integer) oRanking).intValue();
-                } else {
-                    try {
-                        ranking = Integer.parseInt(oRanking.toString().trim());
-                    } catch (final NumberFormatException e) {
-                        ranking = defaultRanking;
-                    }
-                }
-            }
-        }
-        return ranking;
-    }
-
     // ------------------------------------------------------------------------------------------------------------------------------- //
 
     private static class WrappingFilter implements Filter {
