@@ -91,6 +91,34 @@
 
 package com.openexchange.http.grizzly.service.http;
 
+import java.io.IOException;
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.nio.charset.Charset;
+import java.nio.charset.CharsetEncoder;
+import java.util.ArrayList;
+import java.util.Dictionary;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Hashtable;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.TreeSet;
+import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.locks.ReentrantLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
+import javax.servlet.Filter;
+import javax.servlet.Servlet;
+import javax.servlet.ServletException;
+import org.glassfish.grizzly.http.server.ErrorPageGenerator;
+import org.glassfish.grizzly.http.server.HttpHandler;
+import org.glassfish.grizzly.http.server.OXErrorPageGenerator;
+import org.glassfish.grizzly.http.server.Request;
+import org.glassfish.grizzly.http.server.Response;
+import org.glassfish.grizzly.http.server.util.MappingData;
+import org.glassfish.grizzly.http.util.HttpStatus;
 import org.glassfish.grizzly.servlet.FilterRegistration;
 import org.osgi.framework.Bundle;
 import org.osgi.service.http.HttpContext;
@@ -99,26 +127,6 @@ import org.osgi.service.http.NamespaceException;
 import com.openexchange.exception.ExceptionUtils;
 import com.openexchange.log.LogProperties;
 import com.openexchange.marker.OXThreadMarker;
-import javax.servlet.Filter;
-import javax.servlet.Servlet;
-import javax.servlet.ServletException;
-import java.io.IOException;
-import java.nio.ByteBuffer;
-import java.nio.CharBuffer;
-import java.nio.charset.Charset;
-import java.nio.charset.CharsetEncoder;
-import java.util.*;
-import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.concurrent.locks.ReentrantLock;
-import java.util.concurrent.locks.ReentrantReadWriteLock;
-import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
-import org.glassfish.grizzly.http.server.ErrorPageGenerator;
-import org.glassfish.grizzly.http.server.HttpHandler;
-import org.glassfish.grizzly.http.server.OXErrorPageGenerator;
-import org.glassfish.grizzly.http.server.Request;
-import org.glassfish.grizzly.http.server.Response;
-import org.glassfish.grizzly.http.server.util.MappingData;
-import org.glassfish.grizzly.http.util.HttpStatus;
 
 /**
  * OSGi Main HttpHandler.
@@ -162,16 +170,22 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
     private final OSGiCleanMapper mapper;
     private final HttpStatus shutDownStatus;
     private final ErrorPageGenerator errorPageGenerator;
+    private final boolean supportHierachicalLookupOnNotFound;
+    private final int sessionTimeoutInSeconds;
 
     /**
      * Constructor.
      *
      * @param initialFilters The initial Servlet filter to apply
+     * @param supportHierachicalLookupOnNotFound <code>true</code> to support hierarchical look-up of "parent" servlets; otherwise <code>false</code>
+     * @param sessionTimeoutInSeconds The session timeout in seconds
      * @param bundle Bundle that we create if for, for local data reference.
      */
-    public OSGiMainHandler(List<FilterAndPath> initialFilters, Bundle bundle) {
+    public OSGiMainHandler(List<FilterAndPath> initialFilters, boolean supportHierachicalLookupOnNotFound, int sessionTimeoutInSeconds, Bundle bundle) {
         super();
         this.initialFilters = initialFilters;
+        this.supportHierachicalLookupOnNotFound = supportHierachicalLookupOnNotFound;
+        this.sessionTimeoutInSeconds = sessionTimeoutInSeconds;
         this.bundle = bundle;
         this.mapper = new OSGiCleanMapper();
         this.shutDownStatus = HttpStatus.newHttpStatus(HttpStatus.SERVICE_UNAVAILABLE_503.getStatusCode(), "Server shutting down...");
@@ -189,29 +203,93 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         String alias = request.getDecodedRequestURI();
         String originalAlias = alias;
         LOG.debug("Serviceing URI: {}", alias);
-        // first lookup needs to be done for full match.
-        boolean cutOff = false;
-        while (true) {
-            LOG.debug("CutOff: {}, alias: {}", cutOff, alias);
-            alias = OSGiCleanMapper.map(alias, cutOff);
-            if (alias == null) {
-                if (cutOff) {
-                    // not found
-                    break;
+
+        if (supportHierachicalLookupOnNotFound) {
+            // first lookup needs to be done for full match.
+            boolean cutOff = false;
+            while (true) {
+                LOG.debug("CutOff: {}, alias: {}", cutOff, alias);
+                alias = OSGiCleanMapper.map(alias, cutOff);
+                if (alias == null) {
+                    if (cutOff) {
+                        // not found
+                        break;
+                    }
+                    // switching to reducing mapping mode (removing after last '/' and searching)
+                    LOG.debug("Switching to reducing mapping mode.");
+                    cutOff = true;
+                    alias = originalAlias;
+                } else {
+                    if (SHUTDOWN_REQUESTED.get()) {
+                        // 503 - Service Unavailable
+                        try {
+                            setStatusAndWriteErrorPage(shutDownStatus, null, request, response);
+                        } catch (Exception e) {
+                            LOG.warn("Failed to commit 503 status.", e);
+                        }
+                        return;
+                    }
+
+                    HttpHandler httpHandler = OSGiCleanMapper.getHttpHandler(alias);
+
+                    ReadLock processingLock = ((OSGiHandler) httpHandler).getProcessingLock();
+                    processingLock.lock();
+                    try {
+                        updateMappingInfo(request, alias, originalAlias);
+
+                        OXThreadMarker threadMarker = threadMarker();
+                        threadMarker.setHttpRequestProcessing(true);
+                        try {
+                            httpHandler.service(request, response);
+                        } finally {
+                            threadMarker.setHttpRequestProcessing(false);
+                        }
+                    } catch (Throwable t) {
+                        ExceptionUtils.handleThrowable(t);
+                        StringBuilder logBuilder = new StringBuilder(128).append("Error processing request:\n");
+                        logBuilder.append(LogProperties.getAndPrettyPrint(LogProperties.Name.SESSION_SESSION));
+                        appendRequestInfo(logBuilder, request);
+                        LOG.error(logBuilder.toString(), t);
+                        // 500 - Internal Server Error
+                        response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR_500);
+                    } finally {
+                        processingLock.unlock();
+                    }
+                    invoked = true;
+                    if (response.getStatus() != 404) {
+                        break;
+                    } else if ("/".equals(alias)) {
+                        // 404 in "/", cutoff algo will not escape this one.
+                        break;
+                    } else if (!cutOff){
+                        // not found and haven't run in cutoff mode
+                        cutOff = true;
+                    }
                 }
+            }
+        } else {
+            if (SHUTDOWN_REQUESTED.get()) {
+                // 503 - Service Unavailable
+                try {
+                    setStatusAndWriteErrorPage(shutDownStatus, null, request, response);
+                } catch (Exception e) {
+                    LOG.warn("Failed to commit 503 status.", e);
+                }
+                return;
+            }
+
+            // first lookup needs to be done for full match.
+            HttpHandler httpHandler = OSGiCleanMapper.map2HttpHandler(alias);
+            if (httpHandler == null) {
                 // switching to reducing mapping mode (removing after last '/' and searching)
                 LOG.debug("Switching to reducing mapping mode.");
-                cutOff = true;
-                alias = originalAlias;
-            } else {
-                if (SHUTDOWN_REQUESTED.get()) {
-                    // 503 - Service Unavailable
-                    response.setStatus(shutDownStatus);
-                    return;
+                HttpHandlerMatch match = OSGiCleanMapper.map2HttpHandler(alias, true);
+                if (null != match) {
+                    alias = match.alias;
+                    httpHandler = match.httpHandler;
                 }
-
-                HttpHandler httpHandler = OSGiCleanMapper.getHttpHandler(alias);
-
+            }
+            if (null != httpHandler) {
                 ReadLock processingLock = ((OSGiHandler) httpHandler).getProcessingLock();
                 processingLock.lock();
                 try {
@@ -231,26 +309,21 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
                     appendRequestInfo(logBuilder, request);
                     LOG.error(logBuilder.toString(), t);
                     // 500 - Internal Server Error
-                    response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR_500);
+                    try {
+                        setStatusAndWriteErrorPage(HttpStatus.INTERNAL_SERVER_ERROR_500, null, request, response);
+                    } catch (Exception e) {
+                        LOG.warn("Failed to commit 500 status.", e);
+                    }
                 } finally {
                     processingLock.unlock();
                 }
                 invoked = true;
-                if (response.getStatus() != 404) {
-                    break;
-                } else if ("/".equals(alias)) {
-                    // 404 in "/", cutoff algo will not escape this one.
-                    break;
-                } else if (!cutOff){
-                    // not found and haven't run in cutoff mode
-                    cutOff = true;
-                }
             }
         }
+
         if (!invoked) {
-            response.setStatus(HttpStatus.NOT_FOUND_404);
             try {
-                writeErrorPage(request, response);
+                setStatusAndWriteErrorPage(HttpStatus.NOT_FOUND_404, "Resource does not exist.", request, response);
             } catch (Exception e) {
                 LOG.warn("Failed to commit 404 status.", e);
             }
@@ -535,9 +608,9 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         ReentrantLock lock = OSGiCleanMapper.getLock();
         lock.lock();
         try {
-            Set<String> aliases = OSGiCleanMapper.getAllAliases();
+            TreeSet<String> aliases = (TreeSet<String>) OSGiCleanMapper.getAllAliases();
             while (!aliases.isEmpty()) {
-                String alias = ((TreeSet<String>) aliases).first();
+                String alias = aliases.first();
                 LOG.debug("Unregistering '{}'", alias);
                 // remember not to call Servlet.destroy() owning bundle might be stopped already.
                 mapper.doUnregister(alias, false);
@@ -646,24 +719,28 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
             HashMap<String, String> params;
             if (initparams != null) {
                 params = new HashMap<String, String>(initparams.size());
-                Enumeration names = initparams.keys();
-                while (names.hasMoreElements()) {
-                    String name = (String) names.nextElement();
-                    params.put(name, (String) initparams.get(name));
+                if (initparams instanceof Hashtable) {
+                    @SuppressWarnings("unchecked")
+                    Hashtable<String, String> ht = (Hashtable<String, String>) initparams;
+                    for (Map.Entry<String, String> entry : ht.entrySet()) {
+                        params.put(entry.getKey(), entry.getValue());
+                    }
+                } else {
+                    for (Enumeration<?> names = initparams.keys(); names.hasMoreElements();) {
+                        String name = (String) names.nextElement();
+                        params.put(name, (String) initparams.get(name));
+                    }
                 }
             } else {
                 params = new HashMap<String, String>(0);
             }
 
             servletHandlers = new ArrayList<OSGiServletHandler>(1);
-            mapper.addContext(httpContext,
-                    mapper.getServletContext(httpContext),
-                    servletHandlers);
-
-            final OSGiServletContext servletContext =
-                    mapper.getServletContext(httpContext);
-
-            assert servletContext != null;
+            OSGiServletContext servletContext = mapper.getServletContext(httpContext);
+            if (servletContext == null) {
+                servletContext = new OSGiServletContext(httpContext, sessionTimeoutInSeconds);
+            }
+            mapper.addContext(httpContext, servletContext, servletHandlers);
 
             osgiServletHandler =
                     new OSGiServletHandler(servlet,
@@ -705,9 +782,9 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         return errorPageGenerator;
     }
 
-    private void writeErrorPage(Request req, Response res) throws Exception {
-        res.setStatus(HttpStatus.NOT_FOUND_404);
-        final ByteBuffer bb = getErrorPage(404, "Not found", "Resource does not exist.");
+    private void setStatusAndWriteErrorPage(HttpStatus status, String optDescription, Request req, Response res) throws Exception {
+        res.setStatus(status);
+        final ByteBuffer bb = getErrorPage(status.getStatusCode(), status.getReasonPhrase(), null == optDescription ? status.getReasonPhrase() : optDescription);
         res.setContentLength(bb.limit());
         res.setContentType("text/html");
         org.glassfish.grizzly.http.io.OutputBuffer out = res.getOutputBuffer();
